@@ -37,6 +37,43 @@
 #include "hns_roce_u_db.h"
 #include "hns_roce_u_hw_v2.h"
 
+#define HR_IBV_OPC_MAP(ib_key, hr_key) \
+		[IBV_WR_ ## ib_key] = HNS_ROCE_WQE_OP_ ## hr_key
+
+static const uint32_t hns_roce_opcode[] = {
+	HR_IBV_OPC_MAP(RDMA_WRITE,		RDMA_WRITE),
+	HR_IBV_OPC_MAP(RDMA_WRITE_WITH_IMM,	RDMA_WRITE_WITH_IMM),
+	HR_IBV_OPC_MAP(SEND,			SEND),
+	HR_IBV_OPC_MAP(SEND_WITH_IMM,		SEND_WITH_IMM),
+	HR_IBV_OPC_MAP(RDMA_READ,		RDMA_READ),
+	HR_IBV_OPC_MAP(ATOMIC_CMP_AND_SWP,	ATOMIC_COM_AND_SWAP),
+	HR_IBV_OPC_MAP(ATOMIC_FETCH_AND_ADD,	ATOMIC_FETCH_AND_ADD),
+	HR_IBV_OPC_MAP(LOCAL_INV,		LOCAL_INV),
+	HR_IBV_OPC_MAP(BIND_MW,			BIND_MW_TYPE),
+	HR_IBV_OPC_MAP(SEND_WITH_INV,		SEND_WITH_INV),
+};
+
+static inline uint32_t to_hr_opcode(enum ibv_wr_opcode ibv_opcode)
+{
+	if (ibv_opcode >= ARRAY_SIZE(hns_roce_opcode))
+		return HNS_ROCE_WQE_OP_MASK;
+
+	return hns_roce_opcode[ibv_opcode];
+}
+
+static const unsigned int hns_roce_mtu[] = {
+	[IBV_MTU_256] = 256,
+	[IBV_MTU_512] = 512,
+	[IBV_MTU_1024] = 1024,
+	[IBV_MTU_2048] = 2048,
+	[IBV_MTU_4096] = 4096,
+};
+
+static inline unsigned int mtu_enum_to_int(enum ibv_mtu mtu)
+{
+	return hns_roce_mtu[mtu];
+}
+
 static void *get_send_sge_ex(struct hns_roce_qp *qp, int n);
 
 static void set_data_seg_v2(struct hns_roce_v2_wqe_data_seg *dseg,
@@ -61,14 +98,12 @@ static void set_extend_atomic_seg(struct hns_roce_qp *qp,
 }
 
 static int set_atomic_seg(struct hns_roce_qp *qp, struct ibv_send_wr *wr,
-			  unsigned int msg_len, void *dseg,
-			  struct hns_roce_sge_info *sge_info)
+			  void *dseg, struct hns_roce_sge_info *sge_info)
 {
-	struct hns_roce_wqe_atomic_seg *aseg;
+	struct hns_roce_wqe_atomic_seg *aseg = dseg;
+	unsigned int msg_len = sge_info->total_len;
 	unsigned int ext_sg_num;
 
-	aseg = dseg;
-
 	if (msg_len == STANDARD_ATOMIC_U_BYTE_8) {
 		if (wr->opcode == IBV_WR_ATOMIC_CMP_AND_SWP) {
 			aseg->fetchadd_swap_data = htole64(wr->wr.atomic.swap);
@@ -171,7 +206,7 @@ static void hns_roce_v2_handle_error_cqe(struct hns_roce_v2_cqe *cqe,
 
 static struct hns_roce_v2_cqe *get_cqe_v2(struct hns_roce_cq *cq, int entry)
 {
-	return cq->buf.buf + entry * HNS_ROCE_CQE_ENTRY_SIZE;
+	return cq->buf.buf + entry * cq->cqe_size;
 }
 
 static void *get_sw_cqe_v2(struct hns_roce_cq *cq, int n)
@@ -613,7 +648,7 @@ static int hns_roce_u_v2_poll_cq(struct ibv_cq *ibvcq, int ne,
 	if (npolled || err == V2_CQ_POLL_ERR) {
 		mmio_ordered_writes_hack();
 
-		if (cq->flags & HNS_ROCE_SUPPORT_CQ_RECORD_DB)
+		if (cq->flags & HNS_ROCE_CQ_FLAG_RECORD_DB)
 			*cq->set_ci_db =
 				cq->cons_index & DB_PARAM_CQ_CONSUMER_IDX_M;
 		else
@@ -670,6 +705,11 @@ static void set_sge(struct hns_roce_v2_wqe_data_seg *dseg,
 		sge_info->total_len += wr->sg_list[i].length;
 		sge_info->valid_num++;
 
+		if (wr->send_flags & IBV_SEND_INLINE &&
+		    wr->opcode != IBV_WR_ATOMIC_FETCH_AND_ADD &&
+		    wr->opcode != IBV_WR_ATOMIC_CMP_AND_SWP)
+			continue;
+
 		/* No inner sge in UD wqe */
 		if (sge_info->valid_num <= HNS_ROCE_SGE_IN_WQE &&
 		    qp->ibv_qp.qp_type != IBV_QPT_UD) {
@@ -684,93 +724,281 @@ static void set_sge(struct hns_roce_v2_wqe_data_seg *dseg,
 	}
 }
 
+static int fill_ext_sge_inl_data(struct hns_roce_qp *qp,
+				 const struct ibv_send_wr *wr,
+				 struct hns_roce_sge_info *sge_info)
+{
+	unsigned int sge_sz = sizeof(struct hns_roce_v2_wqe_data_seg);
+	void *dseg;
+	int i;
+
+	if (sge_info->total_len > qp->sq.max_gs * sge_sz)
+		return EINVAL;
+
+	dseg = get_send_sge_ex(qp, sge_info->start_idx);
+
+	for (i = 0; i < wr->num_sge; i++) {
+		memcpy(dseg, (void *)(uintptr_t)wr->sg_list[i].addr,
+		       wr->sg_list[i].length);
+		dseg += wr->sg_list[i].length;
+	}
+
+	sge_info->start_idx += DIV_ROUND_UP(sge_info->total_len, sge_sz);
+
+	return 0;
+}
+
+static bool check_inl_data_len(struct hns_roce_qp *qp, unsigned int len)
+{
+	int mtu = mtu_enum_to_int(qp->path_mtu);
+
+	return (len <= qp->max_inline_data && len <= mtu);
+}
+
+static __le32 get_immtdata(enum ibv_wr_opcode opcode, const struct ibv_send_wr *wr)
+{
+	switch (opcode) {
+	case IBV_WR_SEND_WITH_IMM:
+	case IBV_WR_RDMA_WRITE_WITH_IMM:
+		return htole32(be32toh(wr->imm_data));
+	default:
+		return 0;
+	}
+}
+
+static int check_ud_opcode(struct hns_roce_ud_sq_wqe *ud_sq_wqe,
+			   const struct ibv_send_wr *wr)
+{
+	uint32_t ib_op = wr->opcode;
+
+	if (ib_op != IBV_WR_SEND && ib_op != IBV_WR_SEND_WITH_IMM)
+		return EINVAL;
+
+	ud_sq_wqe->immtdata = get_immtdata(ib_op, wr);
+
+	roce_set_field(ud_sq_wqe->rsv_opcode, UD_SQ_WQE_OPCODE_M,
+		       UD_SQ_WQE_OPCODE_S, to_hr_opcode(ib_op));
+
+	return 0;
+}
+
+static int fill_ud_av(struct hns_roce_ud_sq_wqe *ud_sq_wqe,
+		      struct hns_roce_ah *ah)
+{
+	if (unlikely(ah->av.sl > MAX_SERVICE_LEVEL))
+		return EINVAL;
+
+	roce_set_field(ud_sq_wqe->lbi_flow_label, UD_SQ_WQE_SL_M,
+		       UD_SQ_WQE_SL_S, ah->av.sl);
+
+	roce_set_field(ud_sq_wqe->sge_num_pd, UD_SQ_WQE_PD_M,
+		       UD_SQ_WQE_PD_S, to_hr_pd(ah->ibv_ah.pd)->pdn);
+
+	roce_set_field(ud_sq_wqe->tclass_vlan, UD_SQ_WQE_TCLASS_M,
+		       UD_SQ_WQE_TCLASS_S, ah->av.tclass);
+
+	roce_set_field(ud_sq_wqe->tclass_vlan, UD_SQ_WQE_HOPLIMIT_M,
+		       UD_SQ_WQE_HOPLIMIT_S, ah->av.hop_limit);
+
+	roce_set_field(ud_sq_wqe->lbi_flow_label, UD_SQ_WQE_FLOW_LABEL_M,
+		       UD_SQ_WQE_FLOW_LABEL_S, ah->av.flowlabel);
+
+	roce_set_field(ud_sq_wqe->udpspn_rsv, UD_SQ_WQE_UDP_SPN_M,
+		       UD_SQ_WQE_UDP_SPN_S, ah->av.udp_sport);
+
+	memcpy(ud_sq_wqe->dmac, ah->av.mac, ETH_ALEN);
+	ud_sq_wqe->sgid_index = ah->av.gid_index;
+	memcpy(ud_sq_wqe->dgid, ah->av.dgid, HNS_ROCE_GID_SIZE);
+
+	return 0;
+}
+
+static void fill_ud_data_seg(struct hns_roce_ud_sq_wqe *ud_sq_wqe,
+			     struct hns_roce_qp *qp, struct ibv_send_wr *wr,
+			     struct hns_roce_sge_info *sge_info)
+{
+	roce_set_field(ud_sq_wqe->rsv_msg_start_sge_idx,
+		       UD_SQ_WQE_MSG_START_SGE_IDX_M,
+		       UD_SQ_WQE_MSG_START_SGE_IDX_S,
+		       sge_info->start_idx & (qp->ex_sge.sge_cnt - 1));
+
+	set_sge((struct hns_roce_v2_wqe_data_seg *)ud_sq_wqe, qp, wr, sge_info);
+
+	ud_sq_wqe->msg_len = htole32(sge_info->total_len);
+
+	roce_set_field(ud_sq_wqe->sge_num_pd, UD_SQ_WQE_SGE_NUM_M,
+		       UD_SQ_WQE_SGE_NUM_S, sge_info->valid_num);
+}
+
+static int set_ud_wqe(void *wqe, struct hns_roce_qp *qp,
+		      struct ibv_send_wr *wr, int nreq,
+		      struct hns_roce_sge_info *sge_info)
+{
+	struct hns_roce_ah *ah = to_hr_ah(wr->wr.ud.ah);
+	struct hns_roce_ud_sq_wqe *ud_sq_wqe = wqe;
+	int ret = 0;
+
+	memset(ud_sq_wqe, 0, sizeof(*ud_sq_wqe));
+
+	roce_set_bit(ud_sq_wqe->rsv_opcode, UD_SQ_WQE_CQE_S,
+		     !!(wr->send_flags & IBV_SEND_SIGNALED));
+	roce_set_bit(ud_sq_wqe->rsv_opcode, UD_SQ_WQE_SE_S,
+		     !!(wr->send_flags & IBV_SEND_SOLICITED));
+
+	ret = check_ud_opcode(ud_sq_wqe, wr);
+	if (ret)
+		return ret;
+
+	ud_sq_wqe->qkey = htole32(wr->wr.ud.remote_qkey & 0x80000000 ?
+				  qp->qkey : wr->wr.ud.remote_qkey);
+
+	roce_set_field(ud_sq_wqe->rsv_dqpn, UD_SQ_WQE_DQPN_M,
+		       UD_SQ_WQE_DQPN_S, wr->wr.ud.remote_qpn);
+
+	ret = fill_ud_av(ud_sq_wqe, ah);
+	if (ret)
+		return ret;
+
+	fill_ud_data_seg(ud_sq_wqe, qp, wr, sge_info);
+
+	/*
+	 * The pipeline can sequentially post all valid WQEs in wq buf,
+	 * including those new WQEs waiting for doorbell to update the PI again.
+	 * Therefore, the valid bit of WQE MUST be updated after all of fields
+	 * and extSGEs have been written into DDR instead of cache.
+	 */
+	if (qp->flags & HNS_ROCE_QP_CAP_OWNER_DB)
+		udma_to_device_barrier();
+
+	roce_set_bit(ud_sq_wqe->rsv_opcode, UD_SQ_WQE_OWNER_S,
+		     ~(((qp->sq.head + nreq) >> qp->sq.shift) & 0x1));
+
+	return ret;
+}
+
+static int set_rc_inl(struct hns_roce_qp *qp, const struct ibv_send_wr *wr,
+		      struct hns_roce_rc_sq_wqe *rc_sq_wqe,
+		      struct hns_roce_sge_info *sge_info)
+{
+	unsigned int sge_idx = sge_info->start_idx;
+	void *dseg = rc_sq_wqe;
+	int ret;
+	int i;
+
+	if (wr->opcode == IBV_WR_RDMA_READ)
+		return EINVAL;
+
+	if (!check_inl_data_len(qp, sge_info->total_len))
+		return EINVAL;
+
+	dseg += sizeof(struct hns_roce_rc_sq_wqe);
+
+	roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_INLINE_S, 1);
+
+	if (sge_info->total_len <= HNS_ROCE_MAX_RC_INL_INN_SZ) {
+		roce_set_bit(rc_sq_wqe->byte_20, RC_SQ_WQE_BYTE_20_INL_TYPE_S,
+			     0);
+
+		for (i = 0; i < wr->num_sge; i++) {
+			memcpy(dseg, (void *)(uintptr_t)(wr->sg_list[i].addr),
+			       wr->sg_list[i].length);
+			dseg += wr->sg_list[i].length;
+		}
+	} else {
+		roce_set_bit(rc_sq_wqe->byte_20, RC_SQ_WQE_BYTE_20_INL_TYPE_S,
+			     1);
+
+		ret = fill_ext_sge_inl_data(qp, wr, sge_info);
+		if (ret)
+			return ret;
+
+		sge_info->valid_num = sge_info->start_idx - sge_idx;
+
+		roce_set_field(rc_sq_wqe->byte_16, RC_SQ_WQE_BYTE_16_SGE_NUM_M,
+			       RC_SQ_WQE_BYTE_16_SGE_NUM_S,
+			       sge_info->valid_num);
+	}
+
+	return 0;
+}
+
+static void set_bind_mw_seg(struct hns_roce_rc_sq_wqe *wqe,
+			    const struct ibv_send_wr *wr)
+{
+	roce_set_bit(wqe->byte_4, RC_SQ_WQE_BYTE_4_MW_TYPE_S,
+		     wr->bind_mw.mw->type - 1);
+	roce_set_bit(wqe->byte_4, RC_SQ_WQE_BYTE_4_ATOMIC_S,
+		     (wr->bind_mw.bind_info.mw_access_flags &
+		     IBV_ACCESS_REMOTE_ATOMIC) ? 1 : 0);
+	roce_set_bit(wqe->byte_4, RC_SQ_WQE_BYTE_4_RDMA_READ_S,
+		     (wr->bind_mw.bind_info.mw_access_flags &
+		     IBV_ACCESS_REMOTE_READ) ? 1 : 0);
+	roce_set_bit(wqe->byte_4, RC_SQ_WQE_BYTE_4_RDMA_WRITE_S,
+		     (wr->bind_mw.bind_info.mw_access_flags &
+		     IBV_ACCESS_REMOTE_WRITE) ? 1 : 0);
+	wqe->new_rkey = htole32(wr->bind_mw.rkey);
+	wqe->byte_16 = htole32(wr->bind_mw.bind_info.length &
+			       HNS_ROCE_ADDRESS_MASK);
+	wqe->byte_20 = htole32(wr->bind_mw.bind_info.length >>
+			       HNS_ROCE_ADDRESS_SHIFT);
+	wqe->rkey = htole32(wr->bind_mw.bind_info.mr->rkey);
+	wqe->va = htole64(wr->bind_mw.bind_info.addr);
+}
+
+static int check_rc_opcode(struct hns_roce_rc_sq_wqe *wqe,
+			   const struct ibv_send_wr *wr)
+{
+	int ret = 0;
+
+	wqe->immtdata = get_immtdata(wr->opcode, wr);
+
+	switch (wr->opcode) {
+	case IBV_WR_RDMA_READ:
+	case IBV_WR_RDMA_WRITE:
+	case IBV_WR_RDMA_WRITE_WITH_IMM:
+		wqe->va = htole64(wr->wr.rdma.remote_addr);
+		wqe->rkey = htole32(wr->wr.rdma.rkey);
+		break;
+	case IBV_WR_SEND:
+	case IBV_WR_SEND_WITH_IMM:
+		break;
+	case IBV_WR_ATOMIC_CMP_AND_SWP:
+	case IBV_WR_ATOMIC_FETCH_AND_ADD:
+		wqe->rkey = htole32(wr->wr.atomic.rkey);
+		wqe->va = htole64(wr->wr.atomic.remote_addr);
+		break;
+	case IBV_WR_LOCAL_INV:
+		roce_set_bit(wqe->byte_4, RC_SQ_WQE_BYTE_4_SO_S, 1);
+		/* fallthrough */
+	case IBV_WR_SEND_WITH_INV:
+		wqe->inv_key = htole32(wr->invalidate_rkey);
+		break;
+	case IBV_WR_BIND_MW:
+		set_bind_mw_seg(wqe, wr);
+		break;
+	default:
+		ret = EINVAL;
+		break;
+	}
+
+	roce_set_field(wqe->byte_4, RC_SQ_WQE_BYTE_4_OPCODE_M,
+		       RC_SQ_WQE_BYTE_4_OPCODE_S, to_hr_opcode(wr->opcode));
+
+	return ret;
+}
+
 static int set_rc_wqe(void *wqe, struct hns_roce_qp *qp, struct ibv_send_wr *wr,
 		      int nreq, struct hns_roce_sge_info *sge_info)
 {
 	struct hns_roce_rc_sq_wqe *rc_sq_wqe = wqe;
 	struct hns_roce_v2_wqe_data_seg *dseg;
-	int hr_op;
-	int i;
+	int ret;
 
 	memset(rc_sq_wqe, 0, sizeof(struct hns_roce_rc_sq_wqe));
 
-	switch (wr->opcode) {
-	case IBV_WR_RDMA_READ:
-		hr_op = HNS_ROCE_WQE_OP_RDMA_READ;
-		rc_sq_wqe->va = htole64(wr->wr.rdma.remote_addr);
-		rc_sq_wqe->rkey = htole32(wr->wr.rdma.rkey);
-		break;
-	case IBV_WR_RDMA_WRITE:
-		hr_op = HNS_ROCE_WQE_OP_RDMA_WRITE;
-		rc_sq_wqe->va = htole64(wr->wr.rdma.remote_addr);
-		rc_sq_wqe->rkey = htole32(wr->wr.rdma.rkey);
-		break;
-	case IBV_WR_RDMA_WRITE_WITH_IMM:
-		hr_op = HNS_ROCE_WQE_OP_RDMA_WRITE_WITH_IMM;
-		rc_sq_wqe->va = htole64(wr->wr.rdma.remote_addr);
-		rc_sq_wqe->rkey = htole32(wr->wr.rdma.rkey);
-		rc_sq_wqe->immtdata = htole32(be32toh(wr->imm_data));
-		break;
-	case IBV_WR_SEND:
-		hr_op = HNS_ROCE_WQE_OP_SEND;
-		break;
-	case IBV_WR_SEND_WITH_INV:
-		hr_op = HNS_ROCE_WQE_OP_SEND_WITH_INV;
-		rc_sq_wqe->inv_key = htole32(wr->invalidate_rkey);
-		break;
-	case IBV_WR_SEND_WITH_IMM:
-		hr_op = HNS_ROCE_WQE_OP_SEND_WITH_IMM;
-		rc_sq_wqe->immtdata = htole32(be32toh(wr->imm_data));
-		break;
-	case IBV_WR_LOCAL_INV:
-		hr_op = HNS_ROCE_WQE_OP_LOCAL_INV;
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_SO_S, 1);
-		rc_sq_wqe->inv_key = htole32(wr->invalidate_rkey);
-		break;
-	case IBV_WR_BIND_MW:
-		hr_op = HNS_ROCE_WQE_OP_BIND_MW_TYPE;
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_MW_TYPE_S,
-			     wr->bind_mw.mw->type - 1);
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_ATOMIC_S,
-			     (wr->bind_mw.bind_info.mw_access_flags &
-			     IBV_ACCESS_REMOTE_ATOMIC) ? 1 : 0);
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_RDMA_READ_S,
-			     (wr->bind_mw.bind_info.mw_access_flags &
-			     IBV_ACCESS_REMOTE_READ) ? 1 : 0);
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_RDMA_WRITE_S,
-			     (wr->bind_mw.bind_info.mw_access_flags &
-			     IBV_ACCESS_REMOTE_WRITE) ? 1 : 0);
-		rc_sq_wqe->new_rkey = htole32(wr->bind_mw.rkey);
-		rc_sq_wqe->byte_16 = htole32(wr->bind_mw.bind_info.length &
-					     HNS_ROCE_ADDRESS_MASK);
-		rc_sq_wqe->byte_20 = htole32(wr->bind_mw.bind_info.length >>
-					     HNS_ROCE_ADDRESS_SHIFT);
-		rc_sq_wqe->rkey = htole32(wr->bind_mw.bind_info.mr->rkey);
-		rc_sq_wqe->va = htole64(wr->bind_mw.bind_info.addr);
-		break;
-	case IBV_WR_ATOMIC_CMP_AND_SWP:
-		hr_op = HNS_ROCE_WQE_OP_ATOMIC_COM_AND_SWAP;
-		rc_sq_wqe->rkey = htole32(wr->wr.atomic.rkey);
-		rc_sq_wqe->va = htole64(wr->wr.atomic.remote_addr);
-		roce_set_field(rc_sq_wqe->byte_16, RC_SQ_WQE_BYTE_16_SGE_NUM_M,
-			       RC_SQ_WQE_BYTE_16_SGE_NUM_S,
-			       sge_info->valid_num);
-		break;
-	case IBV_WR_ATOMIC_FETCH_AND_ADD:
-		hr_op = HNS_ROCE_WQE_OP_ATOMIC_FETCH_AND_ADD;
-		rc_sq_wqe->rkey = htole32(wr->wr.atomic.rkey);
-		rc_sq_wqe->va = htole64(wr->wr.atomic.remote_addr);
-		roce_set_field(rc_sq_wqe->byte_16, RC_SQ_WQE_BYTE_16_SGE_NUM_M,
-			       RC_SQ_WQE_BYTE_16_SGE_NUM_S,
-			       sge_info->valid_num);
-		break;
-	default:
-		hr_op = HNS_ROCE_WQE_OP_MASK;
-		return EINVAL;
-	}
-
-	roce_set_field(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_OPCODE_M,
-		       RC_SQ_WQE_BYTE_4_OPCODE_S, hr_op);
+	ret = check_rc_opcode(rc_sq_wqe, wr);
+	if (ret)
+		return ret;
 
 	roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_CQE_S,
 		     (wr->send_flags & IBV_SEND_SIGNALED) ? 1 : 0);
@@ -781,16 +1009,13 @@ static int set_rc_wqe(void *wqe, struct hns_roce_qp *qp, struct ibv_send_wr *wr,
 	roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_SE_S,
 		     (wr->send_flags & IBV_SEND_SOLICITED) ? 1 : 0);
 
-	roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_OWNER_S,
-		     ~(((qp->sq.head + nreq) >> qp->sq.shift) & 0x1));
-
 	roce_set_field(rc_sq_wqe->byte_20,
 		       RC_SQ_WQE_BYTE_20_MSG_START_SGE_IDX_M,
 		       RC_SQ_WQE_BYTE_20_MSG_START_SGE_IDX_S,
 		       sge_info->start_idx & (qp->ex_sge.sge_cnt - 1));
 
 	if (wr->opcode == IBV_WR_BIND_MW)
-		return 0;
+		goto wqe_valid;
 
 	wqe += sizeof(struct hns_roce_rc_sq_wqe);
 	dseg = wqe;
@@ -805,24 +1030,26 @@ static int set_rc_wqe(void *wqe, struct hns_roce_qp *qp, struct ibv_send_wr *wr,
 	if (wr->opcode == IBV_WR_ATOMIC_FETCH_AND_ADD ||
 	    wr->opcode == IBV_WR_ATOMIC_CMP_AND_SWP) {
 		dseg++;
-		return set_atomic_seg(qp, wr, le32toh(rc_sq_wqe->msg_len),
-				      dseg, sge_info);
+		ret = set_atomic_seg(qp, wr, dseg, sge_info);
+	} else if (wr->send_flags & IBV_SEND_INLINE) {
+		ret = set_rc_inl(qp, wr, rc_sq_wqe, sge_info);
 	}
 
-	if (wr->send_flags & IBV_SEND_INLINE && sge_info->valid_num) {
-		if (wr->opcode == IBV_WR_RDMA_READ)
-			return EINVAL;
+	if (ret)
+		return ret;
 
-		if (sge_info->total_len > qp->max_inline_data)
-			return EINVAL;
+wqe_valid:
+	/*
+	 * The pipeline can sequentially post all valid WQEs into WQ buffer,
+	 * including new WQEs waiting for the doorbell to update the PI again.
+	 * Therefore, the owner bit of WQE MUST be updated after all fields
+	 * and extSGEs have been written into DDR instead of cache.
+	 */
+	if (qp->flags & HNS_ROCE_QP_CAP_OWNER_DB)
+		udma_to_device_barrier();
 
-		for (i = 0; i < wr->num_sge; i++) {
-			memcpy(dseg, (void *)(uintptr_t)(wr->sg_list[i].addr),
-			       wr->sg_list[i].length);
-			dseg += wr->sg_list[i].length;
-		}
-		roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_INLINE_S, 1);
-	}
+	roce_set_bit(rc_sq_wqe->byte_4, RC_SQ_WQE_BYTE_4_OWNER_S,
+		     ~(((qp->sq.head + nreq) >> qp->sq.shift) & 0x1));
 
 	return 0;
 }
@@ -872,15 +1099,15 @@ int hns_roce_u_v2_post_send(struct ibv_qp *ibvqp, struct ibv_send_wr *wr,
 		switch (ibvqp->qp_type) {
 		case IBV_QPT_RC:
 			ret = set_rc_wqe(wqe, qp, wr, nreq, &sge_info);
-			if (ret) {
-				*bad_wr = wr;
-				goto out;
-			}
 			break;
-		case IBV_QPT_UC:
 		case IBV_QPT_UD:
+			ret = set_ud_wqe(wqe, qp, wr, nreq, &sge_info);
+			break;
 		default:
 			ret = EINVAL;
+		}
+
+		if (ret) {
 			*bad_wr = wr;
 			goto out;
 		}
@@ -895,7 +1122,7 @@ out:
 
 		hns_roce_update_sq_db(ctx, ibvqp->qp_num, qp->sl, qp->sq.head);
 
-		if (qp->flags & HNS_ROCE_SUPPORT_SQ_RECORD_DB)
+		if (qp->flags & HNS_ROCE_QP_CAP_SQ_RECORD_DB)
 			*(qp->sdb) = qp->sq.head & 0xffff;
 	}
 
@@ -992,7 +1219,7 @@ out:
 
 		udma_to_device_barrier();
 
-		if (qp->flags & HNS_ROCE_SUPPORT_RQ_RECORD_DB)
+		if (qp->flags & HNS_ROCE_QP_CAP_RQ_RECORD_DB)
 			*qp->rdb = qp->rq.head & 0xffff;
 		else
 			hns_roce_update_rq_db(ctx, ibvqp->qp_num, qp->rq.head);
@@ -1066,6 +1293,26 @@ static void hns_roce_v2_cq_clean(struct hns_roce_cq *cq, unsigned int qpn,
 	pthread_spin_unlock(&cq->lock);
 }
 
+static void record_qp_attr(struct ibv_qp *qp, struct ibv_qp_attr *attr,
+			   int attr_mask)
+{
+	struct hns_roce_qp *hr_qp = to_hr_qp(qp);
+
+	if (attr_mask & IBV_QP_PORT)
+		hr_qp->port_num = attr->port_num;
+
+	if (attr_mask & IBV_QP_AV)
+		hr_qp->sl = attr->ah_attr.sl;
+
+	if (attr_mask & IBV_QP_QKEY)
+		hr_qp->qkey = attr->qkey;
+
+	if (qp->qp_type == IBV_QPT_UD)
+		hr_qp->path_mtu = IBV_MTU_4096;
+	else if (attr_mask & IBV_QP_PATH_MTU)
+		hr_qp->path_mtu = attr->path_mtu;
+}
+
 static int hns_roce_u_v2_modify_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 				   int attr_mask)
 {
@@ -1103,11 +1350,7 @@ static int hns_roce_u_v2_modify_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 		hns_roce_init_qp_indices(to_hr_qp(qp));
 	}
 
-	if (attr_mask & IBV_QP_PORT)
-		hr_qp->port_num = attr->port_num;
-
-	if (attr_mask & IBV_QP_AV)
-		hr_qp->sl = attr->ah_attr.sl;
+	record_qp_attr(qp, attr, attr_mask);
 
 	return ret;
 }
