@@ -39,7 +39,6 @@
 #include <ccan/ilog.h>
 #include <ccan/minmax.h>
 #include "hns_roce_u.h"
-#include "hns_roce_u_abi.h"
 #include "hns_roce_u_db.h"
 #include "hns_roce_u_hw_v1.h"
 #include "hns_roce_u_hw_v2.h"
@@ -54,24 +53,27 @@ void hns_roce_init_qp_indices(struct hns_roce_qp *qp)
 }
 
 int hns_roce_u_query_device(struct ibv_context *context,
-			    struct ibv_device_attr *attr)
+			    const struct ibv_query_device_ex_input *input,
+			    struct ibv_device_attr_ex *attr, size_t attr_size)
 {
+	struct ib_uverbs_ex_query_device_resp resp;
+	size_t resp_size = sizeof(resp);
 	int ret;
-	struct ibv_query_device cmd;
 	uint64_t raw_fw_ver;
 	unsigned int major, minor, sub_minor;
 
-	ret = ibv_cmd_query_device(context, attr, &raw_fw_ver, &cmd,
-				   sizeof(cmd));
+	ret = ibv_cmd_query_device_any(context, input, attr, attr_size, &resp,
+				       &resp_size);
 	if (ret)
 		return ret;
 
+	raw_fw_ver = resp.base.fw_ver;
 	major = (raw_fw_ver >> 32) & 0xffff;
 	minor = (raw_fw_ver >> 16) & 0xffff;
 	sub_minor = raw_fw_ver & 0xffff;
 
-	snprintf(attr->fw_ver, sizeof(attr->fw_ver), "%d.%d.%03d", major, minor,
-		 sub_minor);
+	snprintf(attr->orig_attr.fw_ver, sizeof(attr->orig_attr.fw_ver),
+		 "%d.%d.%03d", major, minor, sub_minor);
 
 	return 0;
 }
@@ -275,13 +277,12 @@ static int hns_roce_verify_cq(int *cqe, struct hns_roce_context *context)
 	return 0;
 }
 
-static int hns_roce_alloc_cq_buf(struct hns_roce_device *dev,
-				 struct hns_roce_buf *buf, int nent)
+static int hns_roce_alloc_cq_buf(struct hns_roce_cq *cq, int nent)
 {
-	if (hns_roce_alloc_buf(buf,
-			align(nent * HNS_ROCE_CQE_ENTRY_SIZE, dev->page_size),
-			dev->page_size))
-		return -1;
+	int buf_size = hr_hw_page_align(nent * cq->cqe_size);
+
+	if (hns_roce_alloc_buf(&cq->buf, buf_size, HNS_HW_PAGE_SIZE))
+		return ENOMEM;
 
 	return 0;
 }
@@ -291,12 +292,13 @@ struct ibv_cq *hns_roce_u_create_cq(struct ibv_context *context, int cqe,
 				    int comp_vector)
 {
 	struct hns_roce_device *hr_dev = to_hr_dev(context->device);
-	struct hns_roce_create_cq	cmd = {};
-	struct hns_roce_create_cq_resp	resp = {};
-	struct hns_roce_cq		*cq;
-	int				ret;
+	struct hns_roce_context *hr_ctx = to_hr_ctx(context);
+	struct hns_roce_create_cq_resp resp = {};
+	struct hns_roce_create_cq cmd = {};
+	struct hns_roce_cq *cq;
+	int ret;
 
-	if (hns_roce_verify_cq(&cqe, to_hr_ctx(context)))
+	if (hns_roce_verify_cq(&cqe, hr_ctx))
 		return NULL;
 
 	cq = malloc(sizeof(*cq));
@@ -305,6 +307,9 @@ struct ibv_cq *hns_roce_u_create_cq(struct ibv_context *context, int cqe,
 
 	cq->cons_index = 0;
 
+	cq->cqe_size = hr_ctx->cqe_size;
+	cmd.cqe_size = cq->cqe_size;
+
 	if (pthread_spin_init(&cq->lock, PTHREAD_PROCESS_PRIVATE))
 		goto err;
 
@@ -313,7 +318,7 @@ struct ibv_cq *hns_roce_u_create_cq(struct ibv_context *context, int cqe,
 	else
 		cqe = align_queue_size(cqe);
 
-	if (hns_roce_alloc_cq_buf(hr_dev, &cq->buf, cqe))
+	if (hns_roce_alloc_cq_buf(cq, cqe))
 		goto err;
 
 	cmd.buf_addr = (uintptr_t) cq->buf.buf;
@@ -562,12 +567,10 @@ static int hns_roce_verify_qp(struct ibv_qp_init_attr *attr,
 	if (attr->cap.max_recv_wr && attr->cap.max_recv_wr < min_wqe_num)
 		attr->cap.max_recv_wr = min_wqe_num;
 
-	if ((attr->qp_type != IBV_QPT_RC) && (attr->qp_type != IBV_QPT_UD))
-		return EINVAL;
-
-	if ((attr->qp_type == IBV_QPT_RC) &&
-	    (attr->cap.max_inline_data > HNS_ROCE_MAX_INLINE_DATA_LEN))
-		return EINVAL;
+	if (!(attr->qp_type == IBV_QPT_RC ||
+	      (attr->qp_type == IBV_QPT_UD &&
+	       hr_dev->hw_version >= HNS_ROCE_HW_VER3)))
+		return EOPNOTSUPP;
 
 	return 0;
 }
@@ -751,11 +754,9 @@ static void hns_roce_set_qp_params(struct ibv_pd *pd,
 	qp->sq.max_gs = min(ctx->max_sge, qp->sq.max_gs);
 
 	qp->sq_signal_bits = attr->sq_sig_all ? 0 : 1;
-	qp->max_inline_data = HNS_ROCE_MAX_INLINE_DATA_LEN;
 
 	/* update attr for creating qp */
 	attr->cap.max_send_wr = qp->sq.max_post;
-	attr->cap.max_inline_data = qp->max_inline_data;
 }
 
 static int get_sq_db_addr(struct ibv_pd *pd, struct ibv_qp_init_attr *attr,
@@ -823,10 +824,8 @@ struct ibv_qp *hns_roce_u_create_qp(struct ibv_pd *pd,
 	struct hns_roce_create_qp_resp resp = {};
 	struct hns_roce_context *context = to_hr_ctx(pd->context);
 
-	if (hns_roce_verify_qp(attr, context)) {
-		fprintf(stderr, "hns_roce_verify_sizes failed!\n");
+	if (hns_roce_verify_qp(attr, context))
 		return NULL;
-	}
 
 	qp = calloc(1, sizeof(*qp));
 	if (!qp)
@@ -873,6 +872,7 @@ struct ibv_qp *hns_roce_u_create_qp(struct ibv_pd *pd,
 	}
 	pthread_mutex_unlock(&context->qp_table_mutex);
 
+	qp->max_inline_data = attr->cap.max_inline_data;
 	/* adjust rq maxima to not exceed reported device maxima */
 	attr->cap.max_recv_wr = min(context->max_qp_wr, attr->cap.max_recv_wr);
 	attr->cap.max_recv_sge = min(context->max_sge, attr->cap.max_recv_sge);
@@ -930,3 +930,96 @@ int hns_roce_u_query_qp(struct ibv_qp *ibqp, struct ibv_qp_attr *attr,
 
 	return ret;
 }
+
+static uint16_t get_ah_udp_sport(const struct ibv_ah_attr *attr)
+{
+	uint32_t fl = attr->grh.flow_label & IB_GRH_FLOWLABEL_MASK;
+	uint16_t sport;
+
+	if (!fl)
+		sport = get_random() % (IB_ROCE_UDP_ENCAP_VALID_PORT_MAX + 1 -
+					IB_ROCE_UDP_ENCAP_VALID_PORT_MIN) +
+			IB_ROCE_UDP_ENCAP_VALID_PORT_MIN;
+	else
+		sport = ibv_flow_label_to_udp_sport(fl);
+
+	return sport;
+}
+
+static int get_tclass(struct ibv_context *context, struct ibv_ah_attr *attr,
+		      uint8_t *tclass)
+{
+#define DSCP_SHIFT 2
+	enum ibv_gid_type_sysfs gid_type;
+	int ret;
+
+	ret = ibv_query_gid_type(context, attr->port_num, attr->grh.sgid_index,
+				 &gid_type);
+	if (ret)
+		return ret;
+
+	*tclass = gid_type == IBV_GID_TYPE_SYSFS_ROCE_V2 ?
+		  attr->grh.traffic_class >> DSCP_SHIFT :
+		  attr->grh.traffic_class;
+
+	return ret;
+}
+
+struct ibv_ah *hns_roce_u_create_ah(struct ibv_pd *pd, struct ibv_ah_attr *attr)
+{
+	struct hns_roce_device *hr_dev = to_hr_dev(pd->context->device);
+	struct ib_uverbs_create_ah_resp resp = {};
+	struct hns_roce_ah *ah;
+
+	/* HIP08 don't support create ah */
+	if (hr_dev->hw_version < HNS_ROCE_HW_VER3)
+		return NULL;
+
+	ah = malloc(sizeof(*ah));
+	if (!ah)
+		return NULL;
+
+	memset(ah, 0, sizeof(*ah));
+
+	ah->av.port = attr->port_num;
+	ah->av.sl = attr->sl;
+
+	if (attr->is_global) {
+		ah->av.gid_index = attr->grh.sgid_index;
+		ah->av.hop_limit = attr->grh.hop_limit;
+
+		if (get_tclass(pd->context, attr, &ah->av.tclass))
+			goto err;
+
+		ah->av.flowlabel = attr->grh.flow_label;
+
+		memcpy(ah->av.dgid, attr->grh.dgid.raw, ARRAY_SIZE(ah->av.dgid));
+	}
+
+	if (ibv_cmd_create_ah(pd, &ah->ibv_ah, attr, &resp, sizeof(resp)))
+		goto err;
+
+	if (ibv_resolve_eth_l2_from_gid(pd->context, attr, ah->av.mac, NULL))
+		goto err;
+
+	ah->av.udp_sport = get_ah_udp_sport(attr);
+
+	return &ah->ibv_ah;
+
+err:
+	free(ah);
+	return NULL;
+}
+
+int hns_roce_u_destroy_ah(struct ibv_ah *ah)
+{
+	int ret;
+
+	ret = ibv_cmd_destroy_ah(ah);
+	if (ret)
+		return ret;
+
+	free(to_hr_ah(ah));
+
+	return 0;
+}
